package com.winhc.bigdata.spark.ng.jobs

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2020/12/21 09:36
 */
case class inc_company_ng(s: SparkSession,
                          project: String //表所在工程名
                         ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  val ods_tab = "winhc_ng.ods_company"
  val inc_ods_tab = "winhc_ng.inc_ods_company"

  val ads_tab = "winhc_ng.ads_company"
  val inc_ads_tab = "winhc_ng.inc_ads_company"

  def all(): Unit = {
    val inc_ods_ds = getLastPartitionsOrElse(inc_ods_tab, getLastPartitionsOrElse(ods_tab, null))

    if (inc_ods_ds == null) {
      println("ds is null !!!")
      return
    }

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $ads_tab PARTITION(ds='$inc_ods_ds')
         |SELECT  ${getColumns(ads_tab).diff(Seq("ds")).mkString(",")}
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY company_id ORDER BY ds DESC) AS num
         |            FROM    (
         |                        SELECT  *
         |                        FROM    $ods_tab
         |                        WHERE   ds > 0
         |                        UNION ALL
         |                        SELECT  *
         |                        FROM    $inc_ods_tab
         |                        WHERE   ds > 0
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)
  }

  def inc(): Unit = {
    val inc_ods_ds = getLastPartitionsOrElse(inc_ods_tab, getLastPartitionsOrElse(ods_tab, null))
    var ads_ds = getLastPartitionsOrElse(ads_tab, null)
    if (inc_ods_ds.equals(ads_ds)) {
      ads_ds = getSecondLastPartitionOrElse(ads_tab, null)
      if (ads_ds == null) {
        all()
        return
      }
    }

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_ads_tab PARTITION(ds='$inc_ods_ds')
         |SELECT  ${getColumns(inc_ads_tab).diff(Seq("ds")).mkString(",")}
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY company_id ORDER BY ds DESC) AS num
         |            FROM    (
         |                        SELECT  *
         |                        FROM    $inc_ods_tab
         |                        WHERE   ds > $ads_ds
         |                    ) AS t1
         |        ) AS t2
         |WHERE   t2.num = 1
         |""".stripMargin)
  }

  def calc(): Unit = {
    val ads_ds = getLastPartitionsOrElse(ads_tab, null)
    if (ads_ds == null)
      all()
    else
      inc()
  }

}

object inc_company_ng {
  def main(args: Array[String]): Unit = {
    val project = "winhc_ng"
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    inc_company_ng(s = spark, project = project).calc()

    spark.stop()
  }
}
